@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020-2021 Nicotine+ Team
+# COPYRIGHT (C) 2020-2022 Nicotine+ Team
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -21,13 +21,14 @@ import os
 from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import Gtk
+from gi.repository import Pango
 
 
 """ File Choosers """
 
 
 # We need to keep a reference to GtkFileChooserNative, as GTK does not keep it alive
-active_chooser = None
+ACTIVE_CHOOSER = None
 
 
 def _on_selected(dialog, response_id, callback, callback_data):
@@ -70,26 +71,28 @@ def _set_filechooser_folder(dialog, folder_path):
 
 def choose_dir(parent, callback, callback_data=None, initialdir="~", title=_("Select a Folder"), multichoice=True):
 
+    global ACTIVE_CHOOSER  # pylint:disable=global-statement
     try:
-        self = Gtk.FileChooserNative.new(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserNative(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.SELECT_FOLDER
         )
     except AttributeError:
-        self = Gtk.FileChooserDialog(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserDialog(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.SELECT_FOLDER
         )
         self.add_buttons(_("_Cancel"), Gtk.ResponseType.CANCEL, _("_Open"), Gtk.ResponseType.ACCEPT)
 
-    global active_chooser
-    active_chooser = self
-
     self.connect("response", _on_selected, callback, callback_data)
     self.set_modal(True)
 
+    if Gtk.get_major_version() == 3:
+        # Display network shares
+        self.set_local_only(False)
+
     if multichoice:
         self.set_select_multiple(True)
 
@@ -99,27 +102,29 @@ def choose_dir(parent, callback, callback_data=None, initialdir="~", title=_("Se
 
 def choose_file(parent, callback, callback_data=None, initialdir="~", title=_("Select a File"), multiple=False):
 
+    global ACTIVE_CHOOSER  # pylint:disable=global-statement
     try:
-        self = Gtk.FileChooserNative.new(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserNative(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.OPEN
         )
     except AttributeError:
-        self = Gtk.FileChooserDialog(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserDialog(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.OPEN
         )
         self.add_buttons(_("_Cancel"), Gtk.ResponseType.CANCEL, _("_Open"), Gtk.ResponseType.ACCEPT)
 
-    global active_chooser
-    active_chooser = self
-
     self.connect("response", _on_selected, callback, callback_data)
     self.set_modal(True)
     self.set_select_multiple(multiple)
 
+    if Gtk.get_major_version() == 3:
+        # Display network shares
+        self.set_local_only(False)
+
     _set_filechooser_folder(self, initialdir)
     self.show()
 
@@ -130,43 +135,45 @@ def choose_image(parent, callback, callback_data=None, initialdir="~", title=_("
         path = chooser.get_preview_filename()
 
         try:
-            pixbuf = GdkPixbuf.Pixbuf.new_from_file(path)
+            image_data = GdkPixbuf.Pixbuf.new_from_file(path)
 
             maxwidth, maxheight = 300.0, 700.0
-            width, height = pixbuf.get_width(), pixbuf.get_height()
+            width, height = image_data.get_width(), image_data.get_height()
             scale = min(maxwidth / width, maxheight / height)
 
             if scale < 1:
                 width, height = int(width * scale), int(height * scale)
-                pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
+                image_data = image_data.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
 
-            preview.set_from_pixbuf(pixbuf)
+            preview.set_from_pixbuf(image_data)
             chooser.set_preview_widget_active(True)
 
         except Exception:
             chooser.set_preview_widget_active(False)
 
+    global ACTIVE_CHOOSER  # pylint:disable=global-statement
     try:
-        self = Gtk.FileChooserNative.new(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserNative(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.OPEN
         )
     except AttributeError:
-        self = Gtk.FileChooserDialog(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserDialog(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.OPEN
         )
         self.add_buttons(_("_Cancel"), Gtk.ResponseType.CANCEL, _("_Open"), Gtk.ResponseType.ACCEPT)
 
-    global active_chooser
-    active_chooser = self
-
     self.connect("response", _on_selected, callback, callback_data)
     self.set_modal(True)
 
     if Gtk.get_major_version() == 3:
+        # Display network shares
+        self.set_local_only(False)
+
+        # Image preview
         self.connect("update-preview", on_update_image_preview)
 
         preview = Gtk.Image()
@@ -180,28 +187,30 @@ def choose_image(parent, callback, callback_data=None, initialdir="~", title=_("
 
 def save_file(parent, callback, callback_data=None, initialdir="~", initialfile="", title=None):
 
+    global ACTIVE_CHOOSER  # pylint:disable=global-statement
     try:
-        self = Gtk.FileChooserNative.new(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserNative(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.SAVE
         )
     except AttributeError:
-        self = Gtk.FileChooserDialog(
-            parent=parent,
+        self = ACTIVE_CHOOSER = Gtk.FileChooserDialog(
+            transient_for=parent,
             title=title,
             action=Gtk.FileChooserAction.SAVE
         )
         self.add_buttons(_("_Cancel"), Gtk.ResponseType.CANCEL, _("_Save"), Gtk.ResponseType.ACCEPT)
 
-    global active_chooser
-    active_chooser = self
-
     self.connect("response", _on_selected, callback, callback_data)
     self.set_modal(True)
     self.set_select_multiple(False)
 
     if Gtk.get_major_version() == 3:
+        # Display network shares
+        self.set_local_only(False)
+
+        # Display hidden files
         self.set_show_hidden(True)
 
     _set_filechooser_folder(self, initialdir)
@@ -221,20 +230,19 @@ class FileChooserButton:
         self.selected_function = selected_function
         self.path = ""
 
-        box = Gtk.Box()
-        box.set_spacing(6)
-        self.icon = Gtk.Image.new()
-
         if chooser_type == "folder":
-            self.icon.set_property("icon-name", "folder-symbolic")
+            icon_name = "folder-symbolic"
 
         elif chooser_type == "image":
-            self.icon.set_property("icon-name", "image-x-generic-symbolic")
+            icon_name = "image-x-generic-symbolic"
 
         else:
-            self.icon.set_property("icon-name", "text-x-generic-symbolic")
+            icon_name = "text-x-generic-symbolic"
 
-        self.label = Gtk.Label.new(_("(None)"))
+        self.icon = Gtk.Image(icon_name=icon_name)
+        self.label = Gtk.Label(label=_("(None)"), ellipsize=Pango.EllipsizeMode.END, width_chars=6, xalign=0)
+
+        box = Gtk.Box(spacing=6)
         box.add(self.icon)
         box.add(self.label)
 
@@ -246,7 +254,7 @@ class FileChooserButton:
 
         self.button.connect("clicked", self.open_file_chooser)
 
-    def open_file_chooser_response(self, selected, data):
+    def open_file_chooser_response(self, selected, _data):
 
         self.set_path(selected)
 
@@ -257,7 +265,7 @@ class FileChooserButton:
             # No function defined
             return
 
-    def open_file_chooser(self, *args):
+    def open_file_chooser(self, *_args):
 
         if self.chooser_type == "folder":
             choose_dir(
