@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020-2021 Nicotine+ Team
+# COPYRIGHT (C) 2020-2022 Nicotine+ Team
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -18,43 +18,46 @@
 
 from os.path import commonprefix
 
-from gi.repository import Gdk
 from gi.repository import Gtk
 
 from pynicotine import slskmessages
 from pynicotine.config import config
-from pynicotine.gtkgui.utils import connect_key_press_event
-from pynicotine.gtkgui.utils import get_key_press_event_args
-from pynicotine.gtkgui.utils import parse_accelerator
+from pynicotine.gtkgui.widgets.accelerator import Accelerator
 from pynicotine.logfacility import log
 from pynicotine.utils import add_alias
-from pynicotine.utils import expand_alias
+from pynicotine.utils import get_alias
 from pynicotine.utils import is_alias
 from pynicotine.utils import unalias
 
 
-""" Text Entry/View-related """
+""" Text Entry-related """
 
 
 class ChatEntry:
     """ Custom text entry with support for chat commands and completions """
 
-    def __init__(self, frame, entry, entity, message_class, send_message, command_list, is_chatroom=False):
+    def __init__(self, frame, entry, completion, entity, message_class, send_message, command_list, is_chatroom=False):
 
         self.frame = frame
         self.entry = entry
+        self.completion = completion
         self.entity = entity
         self.message_class = message_class
         self.send_message = send_message
         self.command_list = command_list
         self.is_chatroom = is_chatroom
 
-        self.completion_list = None
-        self.completion_iters = {}
-
         entry.connect("activate", self.on_enter)
-        self.entry_changed_handler = entry.connect("changed", self.on_entry_changed)
-        self.key_controller = connect_key_press_event(entry, self.on_key_press_event)
+        Accelerator("<Shift>Tab", entry, self.on_tab_complete_accelerator, True)
+        Accelerator("Tab", entry, self.on_tab_complete_accelerator)
+
+        # Emoji Picker
+        try:
+            self.entry.set_property("show-emoji-icon", True)
+
+        except TypeError:
+            # GTK version not supported
+            pass
 
         # Spell Check
         if config.sections["ui"]["spellcheck"]:
@@ -62,165 +65,24 @@ class ChatEntry:
                 self.frame.init_spell_checker()
 
             if self.frame.spell_checker:
-                from gi.repository import Gspell
+                from gi.repository import Gspell  # pylint:disable=no-name-in-module
                 spell_buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(entry.get_buffer())
                 spell_buffer.set_spell_checker(self.frame.spell_checker)
                 spell_view = Gspell.Entry.get_from_gtk_entry(entry)
                 spell_view.set_inline_spell_checking(True)
 
-        self.midwaycompletion = False  # True if the user just used tab completion
-        self.completions = {}  # Holds temp. information about tab completoin
+    def on_enter(self, *_args):
 
-        completion = Gtk.EntryCompletion()
-        list_store = Gtk.ListStore(str)
-        completion.set_model(list_store)
-        completion.set_text_column(0)
-        completion.set_match_func(self.entry_completion_find_match)
-        completion.connect("match-selected", self.entry_completion_found_match)
-
-        entry.set_completion(completion)
-        self.column_numbers = list(range(list_store.get_n_columns()))
-
-    def add_completion(self, item):
-
-        if not config.sections["words"]["tab"]:
+        if not self.frame.np.logged_in:
             return
 
-        if item in self.completion_list:
-            return
-
-        self.completion_list.append(item)
-
-        if config.sections["words"]["dropdown"]:
-            model = self.entry.get_completion().get_model()
-            self.completion_iters[item] = model.insert_with_valuesv(-1, self.column_numbers, [item])
-
-    def get_completion(self, part, list):
-
-        matches = self.get_completions(part, list)
-
-        if not matches:
-            return None, 0
-
-        if len(matches) == 1:
-            return matches[0], 1
-        else:
-            return commonprefix([x.lower() for x in matches]), 0
-
-    def get_completions(self, part, list):
-
-        lowerpart = part.lower()
-        matches = [x for x in list if x.lower().startswith(lowerpart) and len(x) >= len(part)]
-        return matches
-
-    def remove_completion(self, item):
-
-        if not config.sections["words"]["tab"]:
-            return
-
-        if item not in self.completion_list:
-            return
-
-        self.completion_list.remove(item)
-
-        if not config.sections["words"]["dropdown"]:
-            return
-
-        model = self.entry.get_completion().get_model()
-        iterator = self.completion_iters[item]
-        model.remove(iterator)
-        del self.completion_iters[item]
-
-    def set_completion_list(self, completion_list):
-
-        config_words = config.sections["words"]
-
-        completion = self.entry.get_completion()
-        completion.set_popup_single_match(not config_words["onematch"])
-        completion.set_minimum_key_length(config_words["characters"])
-        completion.set_inline_completion(False)
-
-        model = completion.get_model()
-        model.clear()
-        self.completion_iters.clear()
-
-        if not config_words["tab"]:
-            return
-
-        if completion_list is None:
-            completion_list = []
-
-        self.completion_list = completion_list
-
-        if not config_words["dropdown"]:
-            completion.set_popup_completion(False)
-            return
-
-        for word in completion_list:
-            word = str(word)
-            self.completion_iters[word] = model.insert_with_valuesv(-1, self.column_numbers, [word])
-
-        completion.set_popup_completion(True)
-
-    def entry_completion_find_match(self, completion, entry_text, iterator):
-
-        if not entry_text:
-            return False
-
-        # Get word to the left of current position
-        if " " in entry_text:
-            ix = self.entry.get_position()
-            split_key = entry_text[:ix].split(" ")[-1]
-        else:
-            split_key = entry_text
-
-        if not split_key or len(split_key) < config.sections["words"]["characters"]:
-            return False
-
-        # Case-insensitive matching
-        item_text = completion.get_model().get_value(iterator, 0).lower()
-
-        if item_text.startswith(split_key) and item_text != split_key:
-            return True
-
-        return False
-
-    def entry_completion_found_match(self, completion, model, iterator):
-
-        current_text = self.entry.get_text()
-        completion_value = model.get_value(iterator, 0)
-
-        # if more than a word has been typed, we throw away the
-        # one to the left of our current position because we want
-        # to replace it with the matching word
-
-        if " " in current_text:
-            ix = self.entry.get_position()
-            prefix = " ".join(current_text[:ix].split(" ")[:-1])
-            suffix = " ".join(current_text[ix:].split(" "))
-
-            # add the matching word
-            new_text = "%s %s%s" % (prefix, completion_value, suffix)
-            # set back the whole text
-            self.entry.set_text(new_text)
-            # move the cursor at the end
-            self.entry.set_position(len(prefix) + len(completion_value) + 1)
-        else:
-            self.entry.set_text(completion_value)
-            self.entry.set_position(-1)
-
-        # stop the event propagation
-        return True
-
-    def on_enter(self, widget):
-
         text = self.entry.get_text()
 
         if not text:
             return
 
         if is_alias(text):
-            alias_text = expand_alias(text)
+            alias_text = get_alias(text)
 
             if not alias_text:
                 log.add(_('Alias "%s" returned nothing'), text)
@@ -228,32 +90,14 @@ class ChatEntry:
 
             text = alias_text
 
-        split = text.split(" ", 1)
-        cmd = split[0]
-        is_double_slash_cmd = cmd.startswith("//")
-        is_single_slash_cmd = (cmd.startswith("/") and not is_double_slash_cmd)
+        is_double_slash_cmd = text.startswith("//")
+        is_single_slash_cmd = (text.startswith("/") and not is_double_slash_cmd)
 
-        # Remove empty items created by split, if command ended with a space, e.g. '/ctcpversion '
-        if len([i for i in split if i]) == 2:
-            arg_self = args = split[1]
-        else:
-            if not self.is_chatroom:
-                arg_self = self.entity
-            else:
-                arg_self = ""
-
-            args = ""
-
-        if is_single_slash_cmd and cmd + " " not in self.command_list:
-            log.add(_("Command %s is not recognized"), text)
-            return
-
-        # Clear chat entry
-        self.entry.set_text("")
-
-        if not is_single_slash_cmd or cmd == "/me":
+        if not is_single_slash_cmd or text.startswith("/me"):
             # Regular chat message (/me is sent as plain text)
 
+            self.entry.set_text("")
+
             if is_double_slash_cmd:
                 # Remove first slash and send the rest of the command as plain text
                 text = text[1:]
@@ -261,6 +105,22 @@ class ChatEntry:
             self.send_message(self.entity, text)
             return
 
+        cmd_split = text.split(maxsplit=1)
+        cmd = cmd_split[0]
+
+        if cmd + " " not in self.command_list:
+            log.add(_("Command %s is not recognized"), cmd)
+            return
+
+        # Clear chat entry
+        self.entry.set_text("")
+
+        if len(cmd_split) == 2:
+            args = arg_self = cmd_split[1]
+        else:
+            args = ""
+            arg_self = "" if self.is_chatroom else self.entity
+
         if cmd in ("/alias", "/al"):
             parent = self.frame.np.chatrooms if self.is_chatroom else self.frame.np.privatechats
             parent.echo_message(self.entity, add_alias(args))
@@ -296,44 +156,38 @@ class ChatEntry:
 
         elif cmd in ("/m", "/msg"):
             if args:
-                s = args.split(" ", 1)
-                user = s[0]
-                if len(s) == 2:
-                    msg = s[1]
-                else:
-                    msg = None
-                self.frame.np.privatechats.show_user(args)
-                self.frame.np.privatechats.send_message(user, msg)
-                self.frame.change_main_page("private")
+                args_split = args.split(" ", maxsplit=1)
+                user = args_split[0]
+                msg = None
+
+                if len(args_split) == 2:
+                    msg = args_split[1]
+
+                if msg:
+                    self.frame.np.privatechats.show_user(user)
+                    self.frame.np.privatechats.send_message(user, msg)
+                    self.frame.change_main_page("private")
 
         elif cmd in ("/s", "/search"):
             if args:
-                self.frame.SearchMethod.set_active(0)
-                self.frame.SearchEntry.set_text(args)
-                self.frame.on_search(self.frame.SearchEntry)
+                self.frame.np.search.do_search(args, "global")
                 self.frame.change_main_page("search")
 
         elif cmd in ("/us", "/usearch"):
-            s = args.split(" ", 1)
-            if len(s) == 2:
-                self.frame.SearchMethod.set_active(3)
-                self.frame.SearchEntry.set_text(s[1])
-                self.frame.UserSearchEntry.set_text(s[0])
-                self.frame.on_search(self.frame.SearchEntry)
+            args_split = args.split(" ", maxsplit=1)
+
+            if len(args_split) == 2:
+                self.frame.np.search.do_search(args_split[1], "user", user=args_split[0])
                 self.frame.change_main_page("search")
 
         elif cmd in ("/rs", "/rsearch"):
             if args:
-                self.frame.SearchMethod.set_active(2)
-                self.frame.SearchEntry.set_text(args)
-                self.frame.on_search(self.frame.SearchEntry)
+                self.frame.np.search.do_search(args, "rooms")
                 self.frame.change_main_page("search")
 
         elif cmd in ("/bs", "/bsearch"):
             if args:
-                self.frame.SearchMethod.set_active(1)
-                self.frame.SearchEntry.set_text(args)
-                self.frame.on_search(self.frame.SearchEntry)
+                self.frame.np.search.do_search(args, "buddies")
                 self.frame.change_main_page("search")
 
         elif cmd in ("/j", "/join"):
@@ -381,40 +235,27 @@ class ChatEntry:
 
         elif cmd in ("/clear", "/cl"):
             if self.is_chatroom:
-                parent = self.frame.chatrooms.joinedrooms[self.entity]
+                parent = self.frame.chatrooms.pages[self.entity]
             else:
-                parent = self.frame.privatechats.users[self.entity]
+                parent = self.frame.privatechat.pages[self.entity]
 
-            parent.on_clear_messages()
+            parent.chat_textview.clear()
 
         elif cmd in ("/a", "/away"):
             self.frame.on_away()
 
         elif cmd in ("/q", "/quit", "/exit"):
             self.frame.on_quit()
-            return  # Avoid gsignal warning
 
         elif cmd in ("/c", "/close"):
-            self.frame.privatechats.users[self.entity].on_close()
+            self.frame.privatechat.pages[self.entity].on_close()
 
         elif cmd == "/now":
             self.frame.np.now_playing.display_now_playing(
                 callback=lambda np_message: self.send_message(self.entity, np_message))
 
         elif cmd == "/rescan":
-            # Rescan public shares if needed
-            if not config.sections["transfers"]["friendsonly"] and config.sections["transfers"]["shared"]:
-                self.frame.on_rescan()
-
-            # Rescan buddy shares if needed
-            if config.sections["transfers"]["enablebuddyshares"]:
-                self.frame.on_buddy_rescan()
-
-        elif cmd in ("/tick", "/t"):
-            self.frame.np.queue.append(slskmessages.RoomTickerSet(self.entity, args))
-
-        elif cmd == "/tickers":
-            self.frame.chatrooms.joinedrooms[self.entity].show_tickers()
+            self.frame.np.shares.rescan_shares()
 
         elif cmd == "/toggle":
             if args:
@@ -426,39 +267,212 @@ class ChatEntry:
         elif not self.is_chatroom:
             self.frame.np.pluginhandler.trigger_private_command_event(self.entity, cmd[1:], args)
 
-    def on_entry_changed(self, *args):
+    def on_tab_complete_accelerator(self, widget, state, backwards=False):
+        """ Tab and Shift+Tab: tab complete chat """
+        return self.completion.on_tab_complete_accelerator(widget, state, backwards)
+
+
+class ChatCompletion:
+
+    def __init__(self):
+
+        self.completion_list = []
+        self.completion_iters = {}
+
+        self.midwaycompletion = False  # True if the user just used tab completion
+        self.completions = {}  # Holds temp. information about tab completoin
+
+        self.entry = None
+        self.entry_changed_handler = None
+        self.entry_completion = None
+        self.model = Gtk.ListStore(str)
+
+        self.column_numbers = list(range(self.model.get_n_columns()))
+
+    def create_entry_completion(self):
+
+        self.entry_completion = Gtk.EntryCompletion(model=self.model)
+        self.entry_completion.set_text_column(0)
+        self.entry_completion.set_match_func(self.entry_completion_find_match)
+        self.entry_completion.connect("match-selected", self.entry_completion_found_match)
+
+    def set_entry(self, entry):
+
+        if self.entry is not None:
+            self.entry.set_completion(None)
+            self.entry.disconnect(self.entry_changed_handler)
+
+        # Reusing an existing GtkEntryCompletion object after unsetting it doesn't work well
+        self.create_entry_completion()
+        entry.set_completion(self.entry_completion)
+
+        self.entry = entry
+        self.entry_changed_handler = entry.connect("changed", self.on_entry_changed)
+
+    def add_completion(self, item):
+
+        if not config.sections["words"]["tab"]:
+            return
+
+        if item in self.completion_list:
+            return
+
+        self.completion_list.append(item)
+
+        if config.sections["words"]["dropdown"]:
+            self.completion_iters[item] = self.model.insert_with_valuesv(-1, self.column_numbers, [item])
+
+    def get_completion(self, part, completion_list):
+
+        matches = self.get_completions(part, completion_list)
+
+        if not matches:
+            return None, 0
+
+        if len(matches) == 1:
+            return matches[0], 1
+
+        return commonprefix([x.lower() for x in matches]), 0
+
+    @staticmethod
+    def get_completions(part, completion_list):
+
+        lowerpart = part.lower()
+        matches = [x for x in completion_list if x.lower().startswith(lowerpart) and len(x) >= len(part)]
+        return matches
+
+    def remove_completion(self, item):
+
+        if not config.sections["words"]["tab"]:
+            return
+
+        if item not in self.completion_list:
+            return
+
+        self.completion_list.remove(item)
+
+        if not config.sections["words"]["dropdown"]:
+            return
+
+        iterator = self.completion_iters[item]
+        self.model.remove(iterator)
+        del self.completion_iters[item]
+
+    def set_completion_list(self, completion_list):
+
+        if self.entry_completion is None:
+            return
+
+        config_words = config.sections["words"]
+
+        self.entry_completion.set_popup_single_match(not config_words["onematch"])
+        self.entry_completion.set_minimum_key_length(config_words["characters"])
+        self.entry_completion.set_inline_completion(False)
+
+        self.model.clear()
+        self.completion_iters.clear()
+
+        if not config_words["tab"]:
+            return
+
+        if completion_list is None:
+            completion_list = []
+
+        self.completion_list = completion_list
+
+        if not config_words["dropdown"]:
+            self.entry_completion.set_popup_completion(False)
+            return
+
+        for word in completion_list:
+            word = str(word)
+            self.completion_iters[word] = self.model.insert_with_valuesv(-1, self.column_numbers, [word])
+
+        self.entry_completion.set_popup_completion(True)
+
+    def entry_completion_find_match(self, _completion, entry_text, iterator):
+
+        if not entry_text:
+            return False
+
+        # Get word to the left of current position
+        if " " in entry_text:
+            i = self.entry.get_position()
+            split_key = entry_text[:i].split(" ")[-1]
+        else:
+            split_key = entry_text
+
+        if not split_key or len(split_key) < config.sections["words"]["characters"]:
+            return False
+
+        # Case-insensitive matching
+        item_text = self.model.get_value(iterator, 0).lower()
+
+        if item_text.startswith(split_key) and item_text != split_key:
+            return True
+
+        return False
+
+    def entry_completion_found_match(self, _completion, model, iterator):
+
+        current_text = self.entry.get_text()
+        completion_value = model.get_value(iterator, 0)
+
+        # if more than a word has been typed, we throw away the
+        # one to the left of our current position because we want
+        # to replace it with the matching word
+
+        if " " in current_text:
+            i = self.entry.get_position()
+            prefix = " ".join(current_text[:i].split(" ")[:-1])
+            suffix = " ".join(current_text[i:].split(" "))
+
+            # add the matching word
+            new_text = "%s %s%s" % (prefix, completion_value, suffix)
+            # set back the whole text
+            self.entry.set_text(new_text)
+            # move the cursor at the end
+            self.entry.set_position(len(prefix) + len(completion_value) + 1)
+        else:
+            self.entry.set_text(completion_value)
+            self.entry.set_position(-1)
+
+        # stop the event propagation
+        return True
+
+    def on_entry_changed(self, *_args):
         # If the entry was modified, and we don't block the handler, we're no longer completing
         self.midwaycompletion = False
 
-    def on_key_press_event(self, *args):
-
-        keyval, keycode, state = get_key_press_event_args(*args)
-        keycodes, mods = parse_accelerator("Tab")
-
-        if keycode not in keycodes:
-            return False
+    def on_tab_complete_accelerator(self, _widget, _state, backwards=False):
+        """ Tab and Shift+Tab: tab complete chat """
 
         if not config.sections["words"]["tab"]:
             return False
 
+        text = self.entry.get_text()
+
+        if not text:
+            return False
+
         # "Hello there Miss<tab> how are you doing"
         # "0  3  6  9  12 15      18 21 24 27 30 33
         #   1  4  7  10 13      16 19 22 25 28 31
         #    2  5  8  11 14      17 20 23 26 29 32
         #
-        # ix = 16
+        # i = 16
         # text = Miss
         # preix = 12
-        ix = self.entry.get_position()
-        text = self.entry.get_text()[:ix].split(" ")[-1]
-        preix = ix - len(text)
+        i = self.entry.get_position()
+        text = text[:i].split(" ")[-1]
+        preix = i - len(text)
 
         if not config.sections["words"]["cycle"]:
             completion, single = self.get_completion(text, self.completion_list)
             if completion:
-                if single and ix == len(text) and not text.startswith("/"):
+                if single and i == len(text) and not text.startswith("/"):
                     completion += ": "
-                self.entry.delete_text(preix, ix)
+                self.entry.delete_text(preix, i)
                 self.entry.insert_text(completion, preix)
                 self.entry.set_position(preix + len(completion))
 
@@ -477,10 +491,10 @@ class ChatEntry:
         if self.midwaycompletion:
             # We're still completing, block handler to avoid modifying midwaycompletion value
             with self.entry.handler_block(self.entry_changed_handler):
-                self.entry.delete_text(ix - len(currentnick), ix)
+                self.entry.delete_text(i - len(currentnick), i)
                 direction = 1  # Forward cycle
 
-                if state & Gdk.ModifierType.SHIFT_MASK:
+                if backwards:
                     direction = -1  # Backward cycle
 
                 self.completions['currentindex'] = ((self.completions['currentindex'] + direction) %
@@ -493,13 +507,44 @@ class ChatEntry:
         return True
 
 
+class CompletionEntry:
+
+    def __init__(self, entry, model, column=0):
+
+        self.entry = entry
+        self.model = model
+        self.column = column
+
+        completion = Gtk.EntryCompletion(inline_completion=True, inline_selection=True,
+                                         popup_single_match=False, model=model)
+        completion.set_text_column(column)
+        completion.set_match_func(self.entry_completion_find_match)
+        entry.set_completion(completion)
+
+    def entry_completion_find_match(self, _completion, entry_text, iterator):
+
+        if not entry_text:
+            return False
+
+        item_text = self.model.get_value(iterator, self.column)
+
+        if not item_text:
+            return False
+
+        if item_text.lower().startswith(entry_text.lower()):
+            return True
+
+        return False
+
+
 class TextSearchBar:
 
-    def __init__(self, textview, search_bar, entry):
+    def __init__(self, textview, search_bar, entry, controller_widget=None, focus_widget=None):
 
         self.textview = textview
         self.search_bar = search_bar
         self.entry = entry
+        self.focus_widget = focus_widget or textview
 
         self.search_bar.connect_entry(self.entry)
 
@@ -509,7 +554,13 @@ class TextSearchBar:
         self.entry.connect("previous-match", self.on_search_previous_match)
         self.entry.connect("next-match", self.on_search_next_match)
 
-        self.key_controller = connect_key_press_event(self.textview, self.on_key_press_event)
+        if not controller_widget:
+            controller_widget = textview
+
+        Accelerator("<Primary>f", controller_widget, self.on_show_search_accelerator)
+
+        for widget in (controller_widget, entry):
+            Accelerator("Escape", widget, self.on_hide_search_accelerator)
 
     def on_search_match(self, search_type, restarted=False):
 
@@ -554,41 +605,31 @@ class TextSearchBar:
 
             self.on_search_match(search_type, restarted=True)
 
-    def on_search_changed(self, *args):
+    def on_search_changed(self, *_args):
         self.on_search_match(search_type="typing")
 
-    def on_search_previous_match(self, *args):
+    def on_search_previous_match(self, *_args):
         self.on_search_match(search_type="previous")
 
-    def on_search_next_match(self, *args):
+    def on_search_next_match(self, *_args):
         self.on_search_match(search_type="next")
 
-    def on_key_press_event(self, *args):
+    def on_hide_search_accelerator(self, *_args):
+        """ Escape: hide search bar """
 
-        keyval, keycode, state = get_key_press_event_args(*args)
-        keycodes, mods = parse_accelerator("<Primary>f")
+        self.hide_search_bar()
+        return True
 
-        if state & mods and keycode in keycodes:
-            self.show_search_bar()
-            return True
+    def on_show_search_accelerator(self, *_args):
+        """ Ctrl+F: show search bar """
 
-        return False
+        self.show_search_bar()
+        return True
 
     def show_search_bar(self):
         self.search_bar.set_search_mode(True)
         self.entry.grab_focus()
 
-
-def clear_entry(entry):
-
-    if Gtk.get_major_version() == 4:
-        completion = entry.get_completion()
-        completion.set_minimum_key_length(1)
-        entry.set_text("")
-        completion.set_minimum_key_length(0)
-
-    else:
-        completion = entry.get_completion()
-        entry.set_completion(None)
-        entry.set_text("")
-        entry.set_completion(completion)
+    def hide_search_bar(self):
+        self.search_bar.set_search_mode(False)
+        self.focus_widget.grab_focus()
