@@ -276,13 +276,16 @@ def get_flag_icon_name(country):
 
 def get_status_icon(status):
 
+    if status == 0:
+        return get_icon("offline")
+
     if status == 1:
         return get_icon("away")
 
     if status == 2:
         return get_icon("online")
 
-    return get_icon("offline")
+    return None
 
 
 def load_ui_icon(name):
@@ -302,7 +305,8 @@ def load_custom_icons(names):
     if not config.sections["ui"].get("icontheme"):
         return False
 
-    log.add_debug("Loading custom icons when available")
+    icon_theme_path = config.sections["ui"]["icontheme"]
+    log.add_debug("Loading custom icon theme from %s", icon_theme_path)
     extensions = ["jpg", "jpeg", "bmp", "png", "svg"]
 
     for name in names:
@@ -311,8 +315,7 @@ def load_custom_icons(names):
         loaded = False
 
         while not path or (exts and not loaded):
-            path = os.path.expanduser(os.path.join(config.sections["ui"]["icontheme"], "%s.%s" %
-                                      (name, exts.pop())))
+            path = os.path.expanduser(os.path.join(icon_theme_path, "%s.%s" % (name, exts.pop())))
 
             try:
                 if os.path.isfile(path):
@@ -371,6 +374,9 @@ def load_icons():
 """ Widget Fonts and Colors """
 
 
+COLOR_RGBA = Gdk.RGBA()
+
+
 def get_user_status_color(status):
 
     if status == 1:
@@ -387,11 +393,9 @@ def parse_color_string(color_string):
     """ Take a color string, e.g. BLUE, and return a HEX color code """
 
     if color_string:
-        color_rgba = Gdk.RGBA()
-
-        if color_rgba.parse(color_string):
+        if COLOR_RGBA.parse(color_string):
             color_hex = "#%02X%02X%02X" % (
-                round(color_rgba.red * 255), round(color_rgba.green * 255), round(color_rgba.blue * 255))
+                round(COLOR_RGBA.red * 255), round(COLOR_RGBA.green * 255), round(COLOR_RGBA.blue * 255))
             return color_hex
 
     return None
