@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020-2021 Nicotine+ Team
+# COPYRIGHT (C) 2020-2022 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2009-2011 Quinox <quinox@users.sf.net>
@@ -456,14 +456,16 @@ class Scanner:
 
 class Shares:
 
-    def __init__(self, core, config, queue, init_shares=True, ui_callback=None):
+    def __init__(self, core, config, queue, network_callback=None, ui_callback=None, init_shares=True):
 
         self.core = core
+        self.network_callback = network_callback
         self.ui_callback = ui_callback
         self.config = config
         self.queue = queue
         self.translatepunctuation = str.maketrans(dict.fromkeys(PUNCTUATION, ' '))
         self.share_dbs = {}
+        self.pending_network_msgs = []
         self.rescanning = False
         self.should_compress_shares = False
         self.compressed_shares_normal = slskmessages.SharedFileList(None, None)
@@ -829,6 +831,11 @@ class Shares:
             self.send_num_shared_folders_files()
 
         self.rescanning = False
+
+        # Process any file transfer queue requests that arrived while scanning
+        if self.network_callback:
+            self.network_callback(self.pending_network_msgs)
+
         return error
 
     """ Quit """
